@@ -1,28 +1,31 @@
 //! @file mp-genie.c
 //! @author J. Marcel van der Veer
-//
+
 //! @section Copyright
-//
-// This file is part of Algol68G - an Algol 68 compiler-interpreter.
-// Copyright 2001-2022 J. Marcel van der Veer <algol68g@xs4all.nl>.
-//
+//!
+//! This file is part of Algol68G - an Algol 68 compiler-interpreter.
+//! Copyright 2001-2024 J. Marcel van der Veer [algol68g@xs4all.nl].
+
 //! @section License
-//
-// This program is free software; you can redistribute it and/or modify it 
-// under the terms of the GNU General Public License as published by the 
-// Free Software Foundation; either version 3 of the License, or 
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
-// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
-// more details. You should have received a copy of the GNU General Public 
-// License along with this program. If not, see <http://www.gnu.org/licenses/>.
+//!
+//! This program is free software; you can redistribute it and/or modify it 
+//! under the terms of the GNU General Public License as published by the 
+//! Free Software Foundation; either version 3 of the License, or 
+//! (at your option) any later version.
+//!
+//! This program is distributed in the hope that it will be useful, but 
+//! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
+//! or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
+//! more details. You should have received a copy of the GNU General Public 
+//! License along with this program. If not, see [http://www.gnu.org/licenses/].
+
+//! @section Synopsis
+//!
+//! Multi-precision interpreter routines.
 
 #include "a68g.h"
 #include "a68g-genie.h"
 #include "a68g-prelude.h"
-#include "a68g-double.h"
 #include "a68g-mp.h"
 
 //! brief LONG REAL long infinity
@@ -48,11 +51,10 @@ void genie_minus_infinity_mp (NODE_T *p)
 void genie_long_max_int (NODE_T * p)
 {
   int digs = DIGITS (M_LONG_INT);
-  int k;
   MP_T *z = nil_mp (p, digs);
   MP_STATUS (z) = (MP_T) INIT_MASK;
   MP_EXPONENT (z) = (MP_T) (digs - 1);
-  for (k = 1; k <= digs; k++) {
+  for (int k = 1; k <= digs; k++) {
     MP_DIGIT (z, k) = (MP_T) (MP_RADIX - 1);
   }
 }
@@ -62,11 +64,10 @@ void genie_long_max_int (NODE_T * p)
 void genie_long_mp_max_int (NODE_T * p)
 {
   int digs = DIGITS (M_LONG_LONG_INT);
-  int k;
   MP_T *z = nil_mp (p, digs);
   MP_STATUS (z) = (MP_T) INIT_MASK;
   MP_EXPONENT (z) = (MP_T) (digs - 1);
-  for (k = 1; k <= digs; k++) {
+  for (int k = 1; k <= digs; k++) {
     MP_DIGIT (z, k) = (MP_T) (MP_RADIX - 1);
   }
 }
@@ -75,11 +76,11 @@ void genie_long_mp_max_int (NODE_T * p)
 
 void genie_long_max_real (NODE_T * p)
 {
-  int k, digs = DIGITS (M_LONG_REAL);
+  unt digs = DIGITS (M_LONG_REAL);
   MP_T *z = nil_mp (p, digs);
   MP_STATUS (z) = (MP_T) INIT_MASK;
   MP_EXPONENT (z) = (MP_T) (MAX_MP_EXPONENT - 1);
-  for (k = 1; k <= digs; k++) {
+  for (int k = 1; k <= digs; k++) {
     MP_DIGIT (z, k) = (MP_T) (MP_RADIX - 1);
   }
 }
@@ -88,11 +89,11 @@ void genie_long_max_real (NODE_T * p)
 
 void genie_long_mp_max_real (NODE_T * p)
 {
-  int k, digs = DIGITS (M_LONG_LONG_REAL);
+  unt digs = DIGITS (M_LONG_LONG_REAL);
   MP_T *z = nil_mp (p, digs);
   MP_STATUS (z) = (MP_T) INIT_MASK;
   MP_EXPONENT (z) = (MP_T) (MAX_MP_EXPONENT - 1);
-  for (k = 1; k <= digs; k++) {
+  for (int k = 1; k <= digs; k++) {
     MP_DIGIT (z, k) = (MP_T) (MP_RADIX - 1);
   }
 }
@@ -145,9 +146,8 @@ void genie_shorten_mp_to_int (NODE_T * p)
 {
   MOID_T *mode = LHS_MODE (p);
   int digs = DIGITS (mode), size = SIZE (mode);
-  MP_T *z;
   DECREMENT_STACK_POINTER (p, size);
-  z = (MP_T *) STACK_TOP;
+  MP_T *z = (MP_T *) STACK_TOP;
   MP_STATUS (z) = (MP_T) INIT_MASK;
   PUSH_VALUE (p, mp_to_int (p, z, digs), A68_INT);
 }
@@ -170,9 +170,8 @@ void genie_shorten_mp_to_real (NODE_T * p)
 {
   MOID_T *mode = LHS_MODE (p);
   int digs = DIGITS (mode), size = SIZE (mode);
-  MP_T *z;
   DECREMENT_STACK_POINTER (p, size);
-  z = (MP_T *) STACK_TOP;
+  MP_T *z = (MP_T *) STACK_TOP;
   MP_STATUS (z) = (MP_T) INIT_MASK;
   PUSH_VALUE (p, mp_to_real (p, z, digs), A68_REAL);
 }
@@ -360,6 +359,13 @@ void genie_sin_mp (NODE_T * p)
   C_L_FUNCTION (p, sin_mp);
 }
 
+//! @brief PROC (LONG REAL) LONG REAL long cas
+
+void genie_cas_mp (NODE_T * p)
+{
+  C_L_FUNCTION (p, cas_mp);
+}
+
 //! @brief PROC (LONG REAL) LONG REAL long cos
 
 void genie_cos_mp (NODE_T * p)
@@ -621,9 +627,8 @@ void genie_pow_mp_int (NODE_T * p)
   MOID_T *mode = LHS_MODE (p);
   int digs = DIGITS (mode), size = SIZE (mode);
   A68_INT k;
-  MP_T *x;
   POP_OBJECT (p, &k, A68_INT);
-  x = (MP_T *) STACK_OFFSET (-size);
+  MP_T *x = (MP_T *) STACK_OFFSET (-size);
   (void) pow_mp_int (p, x, x, VALUE (&k), digs);
   MP_STATUS (x) = (MP_T) INIT_MASK;
 }
@@ -667,49 +672,49 @@ void genie_odd_mp (NODE_T * p)
 
 //! @brief Test whether z is a valid LONG INT.
 
-void test_long_int_range (NODE_T * p, MP_T * z, MOID_T * m)
+void test_mp_int_range (NODE_T * p, MP_T * z, MOID_T * m)
 {
   PRELUDE_ERROR (!check_mp_int (z, m), p, ERROR_OUT_OF_BOUNDS, m);
 }
 
 //! @brief OP + = (LONG INT, LONG INT) LONG INT
 
-void genie_add_long_int (NODE_T * p)
+void genie_add_mp_int (NODE_T * p)
 {
   MOID_T *m = RHS_MODE (p);
   int digs = DIGITS (m), size = SIZE (m);
   MP_T *x = (MP_T *) STACK_OFFSET (-2 * size);
   MP_T *y = (MP_T *) STACK_OFFSET (-size);
   (void) add_mp (p, x, x, y, digs);
-  test_long_int_range (p, x, m);
+  test_mp_int_range (p, x, m);
   MP_STATUS (x) = (MP_T) INIT_MASK;
   DECREMENT_STACK_POINTER (p, size);
 }
 
 //! @brief OP - = (LONG INT, LONG INT) LONG INT
 
-void genie_sub_long_int (NODE_T * p)
+void genie_sub_mp_int (NODE_T * p)
 {
   MOID_T *m = RHS_MODE (p);
   int digs = DIGITS (m), size = SIZE (m);
   MP_T *x = (MP_T *) STACK_OFFSET (-2 * size);
   MP_T *y = (MP_T *) STACK_OFFSET (-size);
   (void) sub_mp (p, x, x, y, digs);
-  test_long_int_range (p, x, m);
+  test_mp_int_range (p, x, m);
   MP_STATUS (x) = (MP_T) INIT_MASK;
   DECREMENT_STACK_POINTER (p, size);
 }
 
 //! @brief OP * = (LONG INT, LONG INT) LONG INT
 
-void genie_mul_long_int (NODE_T * p)
+void genie_mul_mp_int (NODE_T * p)
 {
   MOID_T *m = RHS_MODE (p);
   int digs = DIGITS (m), size = SIZE (m);
   MP_T *x = (MP_T *) STACK_OFFSET (-2 * size);
   MP_T *y = (MP_T *) STACK_OFFSET (-size);
   (void) mul_mp (p, x, x, y, digs);
-  test_long_int_range (p, x, m);
+  test_mp_int_range (p, x, m);
   MP_STATUS (x) = (MP_T) INIT_MASK;
   DECREMENT_STACK_POINTER (p, size);
 }
@@ -721,36 +726,35 @@ void genie_pow_mp_int_int (NODE_T * p)
   MOID_T *m = LHS_MODE (p);
   int digs = DIGITS (m), size = SIZE (m);
   A68_INT k;
-  MP_T *x;
   POP_OBJECT (p, &k, A68_INT);
-  x = (MP_T *) STACK_OFFSET (-size);
+  MP_T *x = (MP_T *) STACK_OFFSET (-size);
   (void) pow_mp_int (p, x, x, VALUE (&k), digs);
-  test_long_int_range (p, x, m);
+  test_mp_int_range (p, x, m);
   MP_STATUS (x) = (MP_T) INIT_MASK;
 }
 
 //! @brief OP +:= = (REF LONG INT, LONG INT) REF LONG INT
 
-void genie_plusab_long_int (NODE_T * p)
+void genie_plusab_mp_int (NODE_T * p)
 {
   MOID_T *mode = LHS_MODE (p);
-  genie_f_and_becomes (p, mode, genie_add_long_int);
+  genie_f_and_becomes (p, mode, genie_add_mp_int);
 }
 
 //! @brief OP -:= = (REF LONG INT, LONG INT) REF LONG INT
 
-void genie_minusab_long_int (NODE_T * p)
+void genie_minusab_mp_int (NODE_T * p)
 {
   MOID_T *mode = LHS_MODE (p);
-  genie_f_and_becomes (p, mode, genie_sub_long_int);
+  genie_f_and_becomes (p, mode, genie_sub_mp_int);
 }
 
 //! @brief OP *:= = (REF LONG INT, LONG INT) REF LONG INT
 
-void genie_timesab_long_int (NODE_T * p)
+void genie_timesab_mp_int (NODE_T * p)
 {
   MOID_T *mode = LHS_MODE (p);
-  genie_f_and_becomes (p, mode, genie_mul_long_int);
+  genie_f_and_becomes (p, mode, genie_mul_mp_int);
 }
 
 //! @brief OP ROUND = (LONG REAL) LONG INT
@@ -767,7 +771,7 @@ void genie_round_mp (NODE_T * p)
 
 #define C_CL_FUNCTION(p, f)\
   MOID_T *mode = MOID (p);\
-  int digs = DIGITSC (mode), size = SIZEC (mode);\
+  int digs = DIGITS_COMPL (mode), size = SIZE_COMPL (mode);\
   ADDR_T pop_sp = A68_SP;\
   MP_T *im = (MP_T *) STACK_OFFSET (-size);\
   MP_T *re = (MP_T *) STACK_OFFSET (-2 * size);\
@@ -935,9 +939,8 @@ void genie_shorten_long_mp_complex_to_mp_complex (NODE_T * p)
   int digs = DIGITS (M_LONG_REAL), size = SIZE (M_LONG_REAL);
   int gdigs = DIGITS (M_LONG_LONG_REAL), size_g = SIZE (M_LONG_LONG_REAL);
   ADDR_T pop_sp = A68_SP;
-  MP_T *a, *b;
-  b = (MP_T *) STACK_OFFSET (-size_g);
-  a = (MP_T *) STACK_OFFSET (-2 * size_g);
+  MP_T *b = (MP_T *) STACK_OFFSET (-size_g);
+  MP_T *a = (MP_T *) STACK_OFFSET (-2 * size_g);
   (void) shorten_mp (p, a, digs, a, gdigs);
   (void) shorten_mp (p, &a[LEN_MP (digs)], digs, b, gdigs);
   A68_SP = pop_sp;
@@ -973,7 +976,7 @@ void genie_im_mp_complex (NODE_T * p)
 
 void genie_minus_mp_complex (NODE_T * p)
 {
-  int size = SIZEC (SUB_MOID (p));
+  int size = SIZE_COMPL (SUB_MOID (p));
   MP_T *b = (MP_T *) STACK_OFFSET (-size);
   MP_T *a = (MP_T *) STACK_OFFSET (-2 * size);
   MP_DIGIT (a, 1) = -MP_DIGIT (a, 1);
@@ -986,7 +989,7 @@ void genie_minus_mp_complex (NODE_T * p)
 
 void genie_conj_mp_complex (NODE_T * p)
 {
-  int size = SIZEC (SUB_MOID (p));
+  int size = SIZE_COMPL (SUB_MOID (p));
   MP_T *b = (MP_T *) STACK_OFFSET (-size);
   MP_T *a = (MP_T *) STACK_OFFSET (-2 * size);
   MP_DIGIT (b, 1) = -MP_DIGIT (b, 1);
@@ -1037,7 +1040,7 @@ void genie_arg_mp_complex (NODE_T * p)
 void genie_add_mp_complex (NODE_T * p)
 {
   MOID_T *mode = SUB_MOID (p);
-  int digs = DIGITSC (mode), size = SIZEC (mode);
+  int digs = DIGITS_COMPL (mode), size = SIZE_COMPL (mode);
   ADDR_T pop_sp = A68_SP;
   MP_T *d = (MP_T *) STACK_OFFSET (-size);
   MP_T *c = (MP_T *) STACK_OFFSET (-2 * size);
@@ -1056,7 +1059,7 @@ void genie_add_mp_complex (NODE_T * p)
 void genie_sub_mp_complex (NODE_T * p)
 {
   MOID_T *mode = SUB_MOID (p);
-  int digs = DIGITSC (mode), size = SIZEC (mode);
+  int digs = DIGITS_COMPL (mode), size = SIZE_COMPL (mode);
   ADDR_T pop_sp = A68_SP;
   MP_T *d = (MP_T *) STACK_OFFSET (-size);
   MP_T *c = (MP_T *) STACK_OFFSET (-2 * size);
@@ -1075,7 +1078,7 @@ void genie_sub_mp_complex (NODE_T * p)
 void genie_mul_mp_complex (NODE_T * p)
 {
   MOID_T *mode = SUB_MOID (p);
-  int digs = DIGITSC (mode), size = SIZEC (mode);
+  int digs = DIGITS_COMPL (mode), size = SIZE_COMPL (mode);
   ADDR_T pop_sp = A68_SP;
   MP_T *d = (MP_T *) STACK_OFFSET (-size);
   MP_T *c = (MP_T *) STACK_OFFSET (-2 * size);
@@ -1093,7 +1096,7 @@ void genie_mul_mp_complex (NODE_T * p)
 void genie_div_mp_complex (NODE_T * p)
 {
   MOID_T *mode = SUB_MOID (p);
-  int digs = DIGITSC (mode), size = SIZEC (mode);
+  int digs = DIGITS_COMPL (mode), size = SIZE_COMPL (mode);
   ADDR_T pop_sp = A68_SP;
   MP_T *d = (MP_T *) STACK_OFFSET (-size);
   MP_T *c = (MP_T *) STACK_OFFSET (-2 * size);
@@ -1111,13 +1114,10 @@ void genie_div_mp_complex (NODE_T * p)
 void genie_pow_mp_complex_int (NODE_T * p)
 {
   MOID_T *mode = SUB_MOID (p);
-  int digs = DIGITSC (mode), size = SIZEC (mode);
-  ADDR_T pop_sp;
+  int digs = DIGITS_COMPL (mode), size = SIZE_COMPL (mode);
   A68_INT j;
-  int expo;
-  BOOL_T negative;
   POP_OBJECT (p, &j, A68_INT);
-  pop_sp = A68_SP;
+  ADDR_T pop_sp = A68_SP;
   MP_T *im_x = (MP_T *) STACK_OFFSET (-size);
   MP_T *re_x = (MP_T *) STACK_OFFSET (-2 * size);
   MP_T *re_z = lit_mp (p, 1, 0, digs);
@@ -1128,12 +1128,12 @@ void genie_pow_mp_complex_int (NODE_T * p)
   (void) move_mp (im_y, im_x, digs);
   MP_T *rea = nil_mp (p, digs);
   MP_T *acc = nil_mp (p, digs);
-  expo = 1;
-  negative = (BOOL_T) (VALUE (&j) < 0);
+  int expo = 1;
+  BOOL_T negative = (BOOL_T) (VALUE (&j) < 0);
   if (negative) {
     VALUE (&j) = -VALUE (&j);
   }
-  while ((unt) expo <= (unt) (VALUE (&j))) {
+  while ((int) expo <= (int) (VALUE (&j))) {
     if (expo & VALUE (&j)) {
       (void) mul_mp (p, acc, im_z, im_y, digs);
       (void) mul_mp (p, rea, re_z, re_y, digs);
@@ -1170,7 +1170,7 @@ void genie_pow_mp_complex_int (NODE_T * p)
 
 void genie_eq_mp_complex (NODE_T * p)
 {
-  int digs = DIGITSC (LHS_MODE (p)), size = SIZEC (LHS_MODE (p));
+  int digs = DIGITS_COMPL (LHS_MODE (p)), size = SIZE_COMPL (LHS_MODE (p));
   ADDR_T pop_sp = A68_SP;
   MP_T *d = (MP_T *) STACK_OFFSET (-size);
   MP_T *c = (MP_T *) STACK_OFFSET (-2 * size);
@@ -1187,7 +1187,7 @@ void genie_eq_mp_complex (NODE_T * p)
 
 void genie_ne_mp_complex (NODE_T * p)
 {
-  int digs = DIGITSC (LHS_MODE (p)), size = SIZEC (LHS_MODE (p));
+  int digs = DIGITS_COMPL (LHS_MODE (p)), size = SIZE_COMPL (LHS_MODE (p));
   ADDR_T pop_sp = A68_SP;
   MP_T *d = (MP_T *) STACK_OFFSET (-size);
   MP_T *c = (MP_T *) STACK_OFFSET (-2 * size);
@@ -1244,98 +1244,126 @@ void genie_long_next_random (NODE_T * p)
   }
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long csc
 
 void genie_csc_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, csc_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long acsc
 
 void genie_acsc_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, acsc_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long sec
 
 void genie_sec_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, sec_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long asec
 
 void genie_asec_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, asec_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long cot
 
 void genie_cot_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, cot_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long acot
 
 void genie_acot_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, acot_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long sindg
 
 void genie_sindg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, sindg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long cosdg
 
 void genie_cosdg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, cosdg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long tandg
 
 void genie_tandg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, tandg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long secdg
+
+void genie_secdg_mp (NODE_T * p)
+{
+  C_L_FUNCTION (p, secdg_mp);
+}
+
+//! @brief PROC (LONG REAL) LONG REAL long asecdg
+
+void genie_asecdg_mp (NODE_T * p)
+{
+  C_L_FUNCTION (p, asecdg_mp);
+}
+
+//! @brief PROC (LONG REAL) LONG REAL long cscdg
+
+void genie_cscdg_mp (NODE_T * p)
+{
+  C_L_FUNCTION (p, cscdg_mp);
+}
+
+//! @brief PROC (LONG REAL) LONG REAL long acscdg
+
+void genie_acscdg_mp (NODE_T * p)
+{
+  C_L_FUNCTION (p, acscdg_mp);
+}
+
+//! @brief PROC (LONG REAL) LONG REAL long cotdg
 
 void genie_cotdg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, cotdg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long asindg
 
 void genie_asindg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, asindg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long acosdg
 
 void genie_acosdg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, acosdg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long atandg
 
 void genie_atandg_mp (NODE_T * p)
 {
   C_L_FUNCTION (p, atandg_mp);
 }
 
-//! @brief PROC (LONG REAL) LONG REAL long
+//! @brief PROC (LONG REAL) LONG REAL long acotdg
 
 void genie_acotdg_mp (NODE_T * p)
 {
